﻿"use strict";

const playerVelocityDeltaX = 0.5;
const playerVelocityMaxX = 20;
const playerVelocityToAngleConst = 0.006;
const gravityAccelY = 0.6;
const playerVelocityMaxY = 85;
const playerInitialJumpVelocityY = -20;
const frictionConst = 0.91;

class Game
{
	constructor(controller, display)
	{
		this.controller = controller;
		this.display = display;

		this.camera = new Camera(this.display);
		this.platforms = new Platforms(this.camera, this.display);

		this.portalColours = [
			"rgba(113, 229, 247, 1)", "rgba(239, 255, 124, 1)", "rgba(249, 159, 239, 1)",
		];

		this.backgroundEarth = new BackgroundEarth(this.camera, this.display);
		this.backgroundBlizzard = new BackgroundBlizzard(this.camera, this.display);
	}

	initialise(level)
	{
		this.platforms.loadMap(level);
		switch (level)
		{
			case Map.EARTH:
				this.background = this.backgroundEarth;
				break;
			case Map.BLIZZARD:
				this.background = this.backgroundBlizzard;
				break;
		}

		this.playerVelocity = new Coord(0, 0);
		this.playerPosition = this.platforms.playerStartPosition.copy();
		this.playerAngle = TAU * 3 / 4;
		this.portalFrame = 0;

		//this.controller.startFrameLoop();
	}

	frame()
	{
		this._update();
		this._render();

		return false;
	}

	_update()
	{
		if (this.platforms.collideWithPortal(this.playerPosition, 40))
		{
			this.controller.setPortalMode(this.platforms.nextLevel);
			return;
		}
		let slowDown = false;
		if (Key.isDown(Key.RIGHT) || Key.isDown(Key.D))
		{
			this.playerVelocity.x += playerVelocityDeltaX;
			if (this.playerVelocity.x > playerVelocityMaxX) {
				this.playerVelocity.x = playerVelocityMaxX;
			}
		}
		else if (this.playerVelocity.x > 0)
			slowDown = true;
		if (Key.isDown(Key.LEFT) || Key.isDown(Key.A))
		{
			this.playerVelocity.x -= playerVelocityDeltaX;
			if (this.playerVelocity.x < -playerVelocityMaxX) {
				this.playerVelocity.x = -playerVelocityMaxX;
			}
		}
		else if (this.playerVelocity.x < 0)
			slowDown = true;
		if (slowDown)
		{
			this.playerVelocity.x *= frictionConst;
			if (Math.abs(this.playerVelocity.x) < 0.03)
				this.playerVelocity.x = 0;
		}
		this.playerPosition.x += this.playerVelocity.x;
		if (this.playerVelocity !== 0 &&
			this.platforms.collideWithMapEdge(this.playerPosition, this.playerVelocity.x, 40))
			this.playerVelocity.x = 0;
		this.playerAngle += this.playerVelocity.x * playerVelocityToAngleConst;
		if (this.playerAngle > TAU)
			this.playerAngle -= TAU;

		if (this.jumpPressed && !this.playerInAir)
		{
			this.playerVelocity.y = playerInitialJumpVelocityY;
			this.playerInAir = true;
		}

		this.playerVelocity.y += gravityAccelY;
		if (this.playerVelocity.y > playerVelocityMaxY)
			this.playerVelocity.y = playerVelocityMaxY;
		this.playerPosition.y += this.playerVelocity.y;
		if (this.playerVelocity.y > 0 &&
			this.platforms.collideWithPlatform(this.playerPosition, this.playerVelocity.y, 40))
		{
			this.playerVelocity.y = 0;
			this.playerInAir = false;
		}
		if (!this.playerInAir && this.playerVelocity.y > gravityAccelY * 2)   // a bit of a hack where you are not in the air if moving very slowly
			this.playerInAir = true;
	}

	onKeyDown(key)
	{
		if (key == Key.UP || key == Key.W)
			this.jumpPressed = true;
	}

	onKeyUp(key)
	{
		if ((key == Key.UP || key == Key.W) && this.jumpPressed)
			this.jumpPressed = false;
	}

	_render()
	{
		this.camera.pos.set(this.platforms.clipCameraPos(this.playerPosition));

		this.background.render();

		this.platforms.render();

		this._renderPortal();
		this._renderPlayer();
	}

	_renderPlayer()
	{
		var pixel = new Coord();
		this.camera.getPixelCoordinatesFromWorldCoordinates(this.playerPosition, pixel);
		this._drawPentagon(this.display.context, pixel, this.playerAngle, 46,
				"rgba(0, 0, 0, 1)");
		//if (!this.playerInAir)
		this._drawPentagon(this.display.context, pixel, this.playerAngle, 41,
				"rgba(255, 33, 73, 1)");
	}

	_drawPentagon(context, pos, angle, radius, colour)
	{
		context.beginPath();
		context.fillStyle = colour;
		context.moveTo(pos.x + Math.cos(angle) * radius, pos.y + Math.sin(angle) * radius);
		for (var i = 1; i <= 5; i++)
		{
			let a = TAU / 5 * i;
			context.lineTo(pos.x + Math.cos(angle + a) * radius, pos.y + Math.sin(angle + a) * radius);
		}
		context.fill();
	}

	_renderPortal()
	{
		var pixel = new Coord();
		this.camera.getPixelCoordinatesFromWorldCoordinates(this.platforms.portalPosition, pixel);
		const frame = Math.floor(this.portalFrame / 6);
		this._drawCircle(this.display.context, pixel, 60, this.portalColours[frame]);
		this._drawCircle(this.display.context, pixel, 40, this.portalColours[(frame + 1) % 3]);
		this._drawCircle(this.display.context, pixel, 20, this.portalColours[(frame + 2) % 3]);
		if (++this.portalFrame == 6 * 3)
			this.portalFrame = 0;
	}

	_drawCircle(context, pos, radius, colour)
	{
		context.fillStyle = colour;
		context.beginPath();
		context.arc(pos.x, pos.y + 20, radius, 0, TAU);
		context.fill();
	}
}