﻿"use strict";

const maxDelta = 3;

const portalColours = [
	[ 249, 109, 87 ],
	[ 249, 221, 107 ],
	[ 210, 249, 124 ],
	[ 149, 249, 191 ],
	[ 137, 249, 236 ],
	[ 149, 223, 249 ],
	[ 189, 169, 249 ],
	[ 249, 137, 150 ]
];

function randomInt(min, max)
{
	return Math.floor(Math.random() * (max - min)) + min;
}

function randomColour()
{
	return randomInt(0, portalColours.length);
}

function randomTime()
{
	return randomInt(20, 30);
}

class Portal
{
	constructor(modeState, display)
	{
		this.modeState = modeState;
		this.display = display;
	}

	initialise()
	{
		this.sections = [];
		for (let i = 0; i < 25; i++)
		{
			let section = {
				c0: randomColour(),
				c1: randomColour(),
				total: randomTime(),
				pos: new Coord(0, 0),
			};
			while (section.c1 === section.c0)
				section.c1 = randomColour();
			section.t = randomInt(0, section.total);
			this.sections.push(section);
		}
		this.arms = [];
		for (let i = 0; i < 5; i++)
		{
			let arm = {
				length: randomInt(50, 60),
				angle: Math.random() * TAU,
				velocity: (randomInt(0, 2) * 2 - 1) * (Math.random() * 0.1 + 0.05),
			};
			this.arms.push(arm);
		}
		this.delta = 0;

		this.end = false;
		var handler = function(p) {
			return function() {
				p.end = true;
			};
		}(this);
		this.timerID = window.setTimeout(handler, 7 * 1000);
	}

	frame()
	{
		if (this.end)
		{
			this.modeState.endWormholeSequence();
			return;
		}
		let armPos = new Coord(0, 0);
		for (const arm of this.arms)
		{
			armPos.x += Math.cos(arm.angle) * arm.length;
			armPos.y += Math.sin(arm.angle) * arm.length;
			arm.angle += arm.velocity;
			if (arm.angle < 0)
				arm.angle += TAU;
			else if (arm.angle >= TAU)
				arm.angle -= TAU;
		}
		this.display.context.save();
		for (let i = 0; i < 25; i++)
		{
			const section = this.sections[i];
			if (i === 24)
				section.pos.set(armPos);
			else
				section.pos.set(this.sections[i + 1].pos);
			if (++section.t >= section.total)
			{
				section.t = 0;
				section.total = randomTime();
				section.c0 = section.c1;
				section.c1 = randomColour();
				while (section.c1 === section.c0)
					section.c1 = randomColour();
			}
			let colour = this._fade(section.c0, section.c1, section.t, section.total);
			if (i >= 25 - 20)
			{
				colour = this._fadeRGB(colour, [255, 255, 255], i - (25 - 20 - 1), 20);
			}
			this.display.context.fillStyle = this.display.rgba(colour);
			const size0 = this.display.size.x / 2 * 4 / (i + 3);
			const size1 = this.display.size.x / 2 * 4 / (i + 2);
			const size = alpha(size0, size1, this.delta / maxDelta);
			this.display.context.beginPath();
			this.display.context.arc(this.display.size.x / 2 + section.pos.x,
					this.display.size.y / 2 + section.pos.y, size, 0, TAU);
			this.display.context.fill();
			this.display.context.clip();
		}
		this.display.context.restore();
		if (++this.delta === maxDelta)
		{
			this.delta = 0;
			for (let i = 0; i < 24; i++)
			{
				const section = this.sections[i];
				const next = this.sections[i + 1];
				section.c0 = next.c0;
				section.c1 = next.c1;
				section.t = next.t;
				section.total = next.total;
			}
			let section = {
				c0: randomColour(),
				c1: randomColour(),
				total: randomTime(),
				pos: this.sections[24].pos,
			};
			while (section.c1 === section.c0)
				section.c1 = randomColour();
			section.t = randomInt(0, section.total);
			this.sections[24] = section;
		}
	}

	_fade(c0, c1, t, total)
	{
		let c = [];
		const a = t / total;
		for (let i = 0; i < 3; i++)
			c[i] = alpha(portalColours[c0][i], portalColours[c1][i], a);
		return c;
	}

	_fadeRGB(c0, c1, t, total)
	{
		let c = [];
		const a = t / total;
		for (let i = 0; i < 3; i++)
			c[i] = alpha(c0[i], c1[i], a);
		return c;
	}
}